/*
  Simple Demo for GLSL
  www.lighthouse3d.com
  
  tweaked a bit by andy
*/

#include "GL/glew.h"

#ifdef __linux__
#include <GL/glut.h>
#endif

#ifdef __APPLE__
#include <glut.h>
#endif

#ifdef _WIN32
#include "glut.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

char redData[12500];
char greenData[12500];
char blueData[12500];
char purpleData[12500];
char yellowData[12500];
char whiteData[12500];
GLuint redtex, greentex, bluetex, yellowtex, whitetex, purpletex;
GLuint texNumNormal;

float mouseWindowX = 0.0;
float mouseWindowY = 0.0;

int windowWidth = 320;
int windowHeight = 320;


////////////////////////////////////////////////////////////////////////////////

char *textFileRead(char *fn)
{
	FILE *fp;
	char *content = NULL;

	int count=0;

	if (fn != NULL) {
		fp = fopen(fn,"rt");

		if (fp != NULL) {
      
      fseek(fp, 0, SEEK_END);
      count = ftell(fp);
      rewind(fp);

			if (count > 0) {
				content = (char *)malloc(sizeof(char) * (count+1));
				count = fread(content,sizeof(char),count,fp);
				content[count] = '\0';
			}
			fclose(fp);
		}
	}
	
	if (content == NULL)
	   {
	   fprintf(stderr, "ERROR: could not load in file %s\n", fn);
	   exit(1);
	   }
	return content;
}           

////////////////////////////////////////////////////////////////////////////////

GLuint p;

float lpos[4] = {1.0, 0.5, 1.0, 0.0};

void changeSize(int w, int h) {

	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;

	float ratio = 1.0* w / h;

	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	// Set the viewport to be the entire window
    glViewport(0, 0, w, h);

    windowWidth = w;
    windowHeight = h;
    
	// Set the correct perspective.
	gluPerspective(45, ratio, 1, 1000);
	glMatrixMode(GL_MODELVIEW);
}

////////////////////////////////////////////////////////////////////////////////

void printShaderLog(GLuint prog)
{
    GLint infoLogLength = 0;
    GLsizei charsWritten  = 0;
    GLchar *infoLog;

    glGetShaderiv(prog, GL_INFO_LOG_LENGTH, &infoLogLength);

    if (infoLogLength > 0)
    {
        infoLog = (char *) malloc(infoLogLength);
        glGetShaderInfoLog(prog, infoLogLength, &charsWritten, infoLog);
		printf("%s\n",infoLog);
        free(infoLog);
    }
}

void printProgramLog(GLuint shad)
{
    GLint infoLogLength = 0;
    GLsizei charsWritten  = 0;
    GLchar *infoLog;

    glGetProgramiv(shad, GL_INFO_LOG_LENGTH, &infoLogLength);

    if (infoLogLength > 0)
    {
        infoLog = (char *) malloc(infoLogLength);
        glGetProgramInfoLog(shad, infoLogLength, &charsWritten, infoLog);
		printf("%s\n",infoLog);
        free(infoLog);
    }
}

////////////////////////////////////////////////////////////////////////////////

float rotAmount = 0;

void renderScene(void) {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	gluLookAt(0.0,3.0,6.0, 
		      0.0,0.0,-1.0,
			  0.0f,1.0f,0.0f);

	glLightfv(GL_LIGHT0, GL_POSITION, lpos);
	glRotatef(rotAmount,0,1,0);

	glRotatef(mouseWindowX*90+90,0,1,0);
	glRotatef(mouseWindowY*90,0,0,1);
	
    glEnable(GL_TEXTURE_CUBE_MAP); 

    glUseProgram(p);

    GLint texLoc, lightLoc;

    lightLoc = glGetUniformLocation(p, "LightPos");
    glUniform3f(lightLoc, 1.0, 1.0, 4.0);
    
    texLoc = glGetUniformLocation(p, "envMap");
    glUniform1i(texLoc, 0); // because we used GL_TEXTURE0 when we bound
    
    glutSolidTeapot(1);

    // rotAmount+= 0.1;

	glutSwapBuffers();
}

////////////////////////////////////////////////////////////////////////////////

void processNormalKeys(unsigned char key, int x, int y) {

	if (key == 27) 
		exit(0);
}

////////////////////////////////////////////////////////////////////////////////

GLuint setShaders(char * vert, char * frag) {
    GLuint v,f, pro;
	char *vs,*fs;

	v = glCreateShader(GL_VERTEX_SHADER);
	f = glCreateShader(GL_FRAGMENT_SHADER);

	vs = textFileRead(vert);
	fs = textFileRead(frag);

	const char * vv = vs;
	const char * ff = fs;

	glShaderSource(v, 1, &vv,NULL);
	glShaderSource(f, 1, &ff,NULL);

	free(vs);free(fs);

	glCompileShader(v);
	glCompileShader(f);

	printShaderLog(v);
	printShaderLog(f);

	pro = glCreateProgram();
	glAttachShader(pro,v);
	glAttachShader(pro,f);

	glLinkProgram(pro);
	printProgramLog(pro);
	
	return(pro);
}

////////////////////////////////////////////////////////////////////////////////

void doTexStuff(char * dataPath, char * fileName, int width, int height,
		char * TheArray)
    {
    char fullPath[256];
    FILE * imageFile;

    strcpy(fullPath, dataPath);
    strcat(fullPath, fileName);
    imageFile = fopen(fullPath, "r");
	
    if (imageFile == NULL) 
	   fprintf(stderr,"Cannot find texture file in data directory");
    else
        {
        fread(TheArray, sizeof(char), height*width*3, imageFile);
        fclose(imageFile);
        }
    }

////////////////////////////////////////////////////////////////////////////////

void PassiveMouseMotion(int x, int y)
{
    if (x<0)
        mouseWindowX = -1;
    else if (x>windowWidth)
        mouseWindowX = 1;
    else
        mouseWindowX = 2* (x / (float) windowWidth) - 1.0;
           
    if (y<0)
        mouseWindowY = -1;
    else if (y>windowHeight)
        mouseWindowY = 1;
    else
        mouseWindowY = 2* (y / (float) windowHeight) - 1.0;
    }

////////////////////////////////////////////////////////////////////////////////

int main(int argc, char **argv) {
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(windowWidth, windowHeight);
	glutCreateWindow("GLSL demo");

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);
	glutReshapeFunc(changeSize);
	glutKeyboardFunc(processNormalKeys);
	glutPassiveMotionFunc(PassiveMouseMotion);

	glewInit();
	
	glEnable(GL_DEPTH_TEST);
	glClearColor(0.0,0.0,0.0,1.0);

    doTexStuff((char *) "./", (char *) "white.raw",  64, 64, whiteData);
    doTexStuff((char *) "./", (char *) "red.raw",    64, 64, redData);
    doTexStuff((char *) "./", (char *) "blue.raw",   64, 64, blueData);
    doTexStuff((char *) "./", (char *) "green.raw",  64, 64, greenData);
    doTexStuff((char *) "./", (char *) "purple.raw", 64, 64, purpleData);
    doTexStuff((char *) "./", (char *) "yellow.raw", 64, 64, yellowData);
         
    GLuint textureNum[1];
    glGenTextures(1, textureNum);
    
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_CUBE_MAP, textureNum[0]); 

    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_REPEAT);
    
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    
    glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X, 
        0, GL_RGB, 64, 64, 0, GL_RGB, GL_UNSIGNED_BYTE, &redData);
    glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X, 
        0, GL_RGB, 64, 64, 0, GL_RGB, GL_UNSIGNED_BYTE, &blueData);
        
    glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y, 
        0, GL_RGB, 64, 64, 0, GL_RGB, GL_UNSIGNED_BYTE, &greenData);
    glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y, 
        0, GL_RGB, 64, 64, 0, GL_RGB, GL_UNSIGNED_BYTE, &yellowData);   
        
    glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z, 
        0, GL_RGB, 64, 64, 0, GL_RGB, GL_UNSIGNED_BYTE, &whiteData);
    glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z, 
        0, GL_RGB, 64, 64, 0, GL_RGB, GL_UNSIGNED_BYTE, &purpleData);


	glewInit();
	if (GLEW_ARB_vertex_shader && GLEW_ARB_fragment_shader)
		printf("Ready for GLSL\n");
	else {
		printf("No GLSL support\n");
		exit(1);
	}

	p = setShaders((char *) "./cube.vert", (char *) "./cube.frag");
	
	glutMainLoop();
	return 0;
}

